"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createImportsTransformer = exports.importContextQueryParam = exports.importQueryParam = void 0;
var typescript_1 = require("typescript");
var path_1 = require("path");
/**
 * @internal
 */
exports.importQueryParam = '__mi';
/**
 * @internal
 */
exports.importContextQueryParam = '__mi_ctx';
/**
 * Create source code transformer
 * @internal
 * @param cwd - working dir
 * @param resolve - dependencies resolver
 * @param file - target file
 * @param preloadList - list modules to preload
 */
function createImportsTransformer(cwd, resolve, file, preloadList) {
    /**
     * Creates import url.
     * @param target - for the dependency
     */
    var url = function (target) {
        var url;
        var resolution = resolve(file, target);
        if (resolution.isUrl) {
            url = resolution.resolution;
        }
        else {
            var resolved = path_1.relative(cwd, resolution.resolution);
            resolved = resolved.replace(/\\/g, '/');
            var context_1 = '';
            while (resolved.startsWith("../")) {
                context_1 += "../";
                resolved = resolved.substr(3);
            }
            url = "/" + resolved + "?" + exports.importQueryParam + "=true" + (context_1 ? "&" + exports.importContextQueryParam + "=" + context_1 : '');
        }
        if (preloadList.indexOf(url) < 0) {
            preloadList.push(url);
        }
        return url;
    };
    return function (context) {
        // creates visitor
        var visitor = function (node) {
            var _a;
            // dynamic import call
            if (typescript_1.isCallExpression(node) && node.expression.kind === typescript_1.SyntaxKind.ImportKeyword) {
                var arg = node.arguments[0];
                if (typescript_1.isStringLiteral(arg)) {
                    return context.factory.updateCallExpression(node, node.expression, node.typeArguments, [
                        context.factory.createStringLiteral(url(arg.text))
                    ]);
                }
                // todo report problematic import
            }
            // export ... from ...
            if (typescript_1.isExportDeclaration(node) && !node.isTypeOnly) {
                var moduleSpecifier = node.moduleSpecifier;
                if (moduleSpecifier && typescript_1.isStringLiteral(moduleSpecifier)) {
                    return context.factory.updateExportDeclaration(node, node.decorators, node.modifiers, node.isTypeOnly, node.exportClause, context.factory.createStringLiteral(url(moduleSpecifier.text)));
                }
            }
            // import ... from ...
            if (typescript_1.isImportDeclaration(node) && !((_a = node.importClause) === null || _a === void 0 ? void 0 : _a.isTypeOnly)) {
                var moduleSpecifier = node.moduleSpecifier;
                if (typescript_1.isStringLiteral(moduleSpecifier)) {
                    return context.factory.updateImportDeclaration(node, node.decorators, node.modifiers, node.importClause, context.factory.createStringLiteral(url(moduleSpecifier.text)));
                }
                return node;
            }
            return typescript_1.visitEachChild(node, visitor, context);
        };
        return function (node) {
            return typescript_1.visitEachChild(node, visitor, context);
        };
    };
}
exports.createImportsTransformer = createImportsTransformer;
//# sourceMappingURL=create-imports-transformer.js.map