import type { ResolveOptions as ResolverResolveOptions } from 'enhanced-resolve';
import type { CustomTransformerFactory, SourceFile, TransformerFactory } from 'typescript';
import { LogLevel } from './logger';
/**
 * Compiler options.
 * @public
 */
export interface CompileOptions {
    /**
     * Path to tsconfig.json file, default is tsconfig.json in working dir
     */
    tsConfigFile?: string;
    /**
     * Overwrites for the tsconfig.json
     */
    compilerOptions?: any;
    /**
     * Custom additional source files transformers
     */
    transformers?: Array<TransformerFactory<SourceFile> | CustomTransformerFactory>;
}
/**
 * Options for node resolver.
 * For more info please go to {@link https://www.npmjs.com/package/enhanced-resolve | enhanced-resolve }
 * documentation.
 * @public
 */
export declare type ResolveOptions = Partial<ResolverResolveOptions>;
/**
 * Value stored in cache.
 * @public
 */
export interface CachedValue {
    /**
     * Last update timestamp.
     */
    timestamp: number;
    /**
     * Transpiled code.
     */
    source: string;
}
/**
 * Caching options.
 * @public
 */
export interface CacheOptions {
    /**
     * Disable cache.
     */
    disabled?: boolean;
    /**
     * Cache provider. By default simple memory cache is used.
     */
    provider?: {
        /**
         * Set value to cache.
         */
        set: (key: string, value: CachedValue) => void;
        /**
         * Get value trom cache.
         */
        get: (key: string) => Promise<CachedValue | undefined>;
    };
}
/**
 * Middleware configuration options.
 * @public
 */
export interface ExpressTypescriptCompileOptions {
    /**
     * Options for compiler.
     */
    compile?: CompileOptions;
    /**
     * Options for the dependencies resolver.
     */
    resolve?: ResolveOptions;
    /**
     * Options for cache.
     */
    cache?: CacheOptions;
    /**
     * Logging level
     */
    logLevel?: LogLevel;
    /**
     * Working dir.
     */
    cwd?: string;
}
/**
 * Prepare the configuration options.
 * @param config - user config
 * @internal
 */
export declare function prepareConfig(config: ExpressTypescriptCompileOptions): Required<ExpressTypescriptCompileOptions>;
//# sourceMappingURL=options.d.ts.map